;
; atapi.asm - lowlevel ATAPI device control
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;---------------------------
;Execute ATAPI command, C=1 if error
;
		
OP_atapi_command_multiplex .proc
		cmp #1
                php
		jsr atapi_select_drive_A
                bcc ok
err             plp
		sec
                rts
ok      				;Set up the controller registers.
		#stz_a low_reg
		sta feat_g		;No OVL/DMA
		sta nsector_g		;No tag

		sta cyllow_g		;Maximum packet size
		lda #>2048
		sta low_reg
		sta cylhi_g

		lda #packet_d		;Packet Command
		jsr ata_data_command
		bcs err

		lda #ATAPI_PH_CMDOUT
		jsr atapi_ireason
		bcs err

		ldx #11
lp		txa
		lsr
		lda packet_buff_exp,x	;Send the command to the device.
		bcs u
		sta hig_reg
		sta data_g
u		sta low_reg
		dex
		bpl lp
                jsr OP_atapi_waitme
                bcs err

                lda #ATAPI_PH_DONE
                plp
                beq atapi_ireason

		lda #%00001000		;DRQ==1?
		bit status_g
                .if FAST_DATA_TRANSFERS=0
		bit low_reg
                .fi
                beq unexp

		#lda0
                rol
                asl		;ATAPI_PH_DATAOUT=CMD_ATAPI_DATAOUT=0, ATAPI_PH_DATAIN=CMD_ATAPI_DATAIN=2
atapi_ireason	.if FAST_DATA_TRANSFERS=0
		bit ireason_g
		eor low_reg
                .else
                eor ireason_g
                .fi
		and #%00000111
unexp		cmp #1
		lda #EUNEXP
		rts
		.pend
