;
; chrout.asm - CHROUT kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Chrout (A=character out)
;
; Used registers: - (all preserved)
; Status: C (N Z for Accu)
; C=1:
; A=7 (not output file)
; Status response:
; ST=$00 if no error
; ST=$03 timeout
; ST=$80 if device not present
;Note:
;Currently buffer is 256 bytes long and only used for error channel...
;**************************************************
newchrout_t	.proc
		sta cfg_open
		.if SUPERCPU=1
		xba
		.else
		sta help_trap_accu_exp
		.fi

		lda buffer_output_exp
		eor #STDOUT
		bne nonstandard
		lda buffer_input_exp
		cmp #STDIN
		bne nonstandard
		#psr gotobank3back
		gcs standard

nonstandard	#psr switch_16k
standard	.if SUPERCPU=1
		xba
		.else
		lda help_trap_accu_exp
		.fi

newchrout	.proc
		pha
		#stx_l_a help_x_exp
		#ldx_l_a buffer_output_exp
		lda buffer_info_exp,x
dback
		.if SUPERCPU=1
		lsr
		bcs chrout_not_data_sector
                
;BLK_WRITE_DIRTY, BLK_RW_DIRTY, BLK_RW_EOF_DIRTY

		lda $a6
		pha
		lda #1
		sta $a6			;SCPU Page 1

hard_chrout	lda buffer_address_low_exp,x
		inc a
		sta buffer_address_low_exp,x
		.else
		asr #$01
		bcs chrout_not_data_sector

hard_chrout	isb buffer_address_low_exp,x
		eor #$ff
		.fi

		beq chrout_nxt_bank
		sta $a4
		lda buffer_address_hi_exp,x
		bmi chrout_data_sector_last
chrout_bnk_nen2	sta $a5
chrout_bnk_nend

		.if SUPERCPU=1		;Store byte
		plx
		pla
		sta [$a4]
		xba
		stx $a6			;Restore $a6
		lda help_x_exp
		tax
		xba			;Set NZ flags
		.else
		pla
		ldx #0
		sta ($a4,x)
		ldx help_x_exp
		ora #$00		;Set NZ flags
		.fi
		jmp exit2basic
;---------------------------------
chrout_nxt_bank	sta $a4
		.if SUPERCPU=1
		lda buffer_address_hi_exp,x
		inc a
		sta buffer_address_hi_exp,x
		.else
		isb buffer_address_hi_exp,x
		eor #$ff
		.fi
		cmp buffer_address_end_hi_exp,x
		bcc chrout_bnk_nen2
		ora #$80
		sta buffer_address_hi_exp,x;Mark next bank

chrout_data_sector_last
		and #$7f
		sta $a5
		lda $a4
		cmp buffer_address_end_low_exp,x
		bcc chrout_bnk_nend
		gcs chrout_next

newconsoljmp	jmp newconsole

;BLK_R_EOF, BLK_RW_EOF, BLK_RW_EOF_FULL, BLK_WRITE_FULL, BLK_WRITE, BLK_READ, BLK_RW, KERNAL, IGNORE

chrout_not_data_sector = *
		cpx #STDOUT
		beq newconsoljmp

		.if SUPERCPU=1
                sec
                rol
                .else
		lda buffer_info_exp,x
                .fi
		cmp #INFO_KERNAL
		bne nchrout_stand

		lda buffer_drive_exp,x
		cmp #4
                .if SUPERCPU=1
                sta ActualOutput
		#ldx_l_a help_x_exp
                .fi
		bcc vew			;serial device?

                .if JIFFYDOS_ACCEL_CHRINOUT=1
                .if SUPERCPU=0
                ldx help_x_exp
                .fi
		jsr OP_cfg16up
		pla
		jsr iecout
                jsr OP_cfg16down
                lda SerialOutBuf
                jmp exit2basic
                .else

                .if ((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)
                lda dos_accel_internal_exp
                beq nx
                .if SUPERCPU=0
                ldx help_x_exp
                .fi
		jsr OP_cfg16up
		pla
		jsr iecout
                jsr OP_cfg16down
                lda SerialOutBuf
                jmp exit2basic
                .fi

nx              pla
		.if SUPERCPU=0
                tax
                .fi
		#psr $eddd
		.if SUPERCPU=0
                txa
                ldx help_x_exp
                .fi
		jmp exit2basic
                .fi		;JIFFYDOS ACCEL CHRINOUT

vew		.if SUPERCPU=0
		sta ActualOutput
                .fi
		#psr $f1cb
		jmp exit2basic

;BLK_R_EOF, BLK_RW_EOF, BLK_RW_EOF_FULL, BLK_WRITE_FULL, BLK_WRITE, BLK_READ, BLK_RW, IGNORE

nchrout_stand	cmp #INFO_BLK_WRITE
		beq brt
		cmp #INFO_BLK_RW
		beq brt2
		cmp #INFO_BLK_RW_EOF
		bne nchrout_err

		lda #INFO_BLK_RW_EOF_DIRTY
		.byte SKIP2
brt2		lda #INFO_BLK_RW_DIRTY
		.byte SKIP2
brt		lda #INFO_BLK_WRITE_DIRTY
		sta buffer_info_exp,x
		gcs dback

nchrout_err	lda #$83		;BLK_R_EOF, BLK_RW_EOF_FULL, BLK_WRITE_FULL, BLK_READ, IGNORE
		sta Status
		#ldx_l_a help_x_exp
		pla
		clc
		jmp exit2basic
;---------------------------------------
chrout_next	#stx_l_a buffer_currentp_exp
		#phy_a
		jsr OP_save_zeropage_scpu01up

		jsr XIT_do_next

		jsr OP_restore_zeropage_scpu01down
		#ply_a
		bcs nchrout_err

		#ldx_l_a buffer_currentp_exp
		.if SUPERCPU=0
		lda #0
		.fi
		gcc hard_chrout
;---------------------------------------
XIT_do_next	#xit_init

		#ldx_l_a buffer_currentp_exp
		lda buffer_type_exp,x
                .if BUFFERMODE_ERROR
                cmp #BUFFERMODE_ERROR
                .fi
                beq XIT_do_errnext
		cmp #BUFFERMODE_DIRECT
		beq XIT_do_direct
		cmp #BUFFERMODE_FILE
		beq XIT_do_filenext
		cmp #BUFFERMODE_PCLINK
		beq XIT_do_pclink
		brk

XIT_do_direct	jsr OP_init_buffer_pointers
yy              pla
                pla
                jsr OP_restore_zeropage_scpu01down
		#ply_a
                clc
		gcc chrout_bnk_nend

XIT_do_pclink	lda buffer_info_exp,x
		pha
                jsr fileopprepare       ;be safe

		lda buffer_address_end_hi_exp,x
		beq vdc
                pla
                pha
                cmp #INFO_STATUS
                beq vdc			;overflow of command buffer?!

		sec
                ldx #<$0800
                ldy #>$0800
		jsr pclink_chrinout
vdc
		jsr OP_init_buffer_pointers
		#jmps chrin_chrout_do_file_end.more
                
XIT_do_errnext	lda #254
		sta buffer_address_low_exp,x
                gne yy
;---------------------------------------
;INFO_BLK_WRITE_DIRTY, INFO_BLK_RW_DIRTY, INFO_BLK_RW_EOF_DIRTY
;
XIT_do_filenext	lda buffer_info_exp,x
		pha
                jsr fileopprepare       ;be safe

		lda buffer_address_end_hi_exp,x
		beq dc
		lda $a5			;buffer_address_hi_exp,x
		cmp buffer_address_max_hi_exp,x
		beq dc2			;eof mark?
                ldy $a4
                cpy #1
                sbc #0
                sta buffer_address_hi_exp,x
                dey
		tya
                sta buffer_address_low_exp,x

		pla
		lda #INFO_BLK_RW_EOF_DIRTY
		pha
		jmp chrin_chrout_do_file_end.more

dc2		jsr XIT_flush_buf

                jsr incfileposhi512

dc		jsr XIT_save_tree	;-=> error

		clc			;no clean
		jsr XIT_read_in_buffer	;-=> error

		jmp chrin_chrout_do_file_end
		.pend

XIT_flush_buf	.proc
		jsr buffer_tree_pom1

		ldy #0
		#lda_zp_y pom1
		and #%01001111
		ldy #3
ur		#ora_zp_y pom1
		bne allre		;is it already allocated?
                dey
                bne ur

		tya
                sec
		adc buffer_address_low_exp,x
		tay
                #lda0
		sta pom1
		adc buffer_address_hi_exp,x
		and #$7f
                cmp buffer_address_max_hi_exp,x
                bcs aad
		sta pom2

vj2		#lda0			;fill with 0
vj		#sta_zp_y pom1
		iny
		bne vj
		inc pom2
		lda pom2
		lsr
		bcs vj2

aad             jsr restore_file_state

                lda #FIND_FREE_DATA
		jsr find_free_space	;allocate data sector

                jsr save_file_state

		jsr buffer_tree_pom1
                lda #$80		;dirty
                sta buffer_treestate_exp,x

		ldy #3
vws		lda next_sector,y
		#sta_zp_y pom1
		dey
		bne vws
		#lda_zp_y pom1		;fill in
		and #%10110000
		ora next_sector
		#sta_zp_y pom1

allre		#jmps XIT_write_out_buffer
		.pend
