;
; dos.asm - Common dos functions
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;---------------------------------------
; hdinit
; C=1 if error then drive_type_exp is 00
;
; fills in drive_type_exp with 01/80/c0
;--------------------------------------
OP_hdinit	.proc
		sta current_drive_exp

		jsr OP_select_common	;Select drive

lp2     	jsr wait_a_bit
		jsr wait_a_bit
		jsr BACK_with_C1_on_timeout_or_C0_status
		lda low_reg
                cmp #$ff
                beq nosig
                asl
		bcs lp2                 ;Wait BSY==0 and DRDY=1

		ldx #0
jf              jsr wait_a_bit
                dex
                bne jf

                lda error_g
                #lda_low_reg
		asl
                cmp #$02		;Check diagnostic code ($01, $81)
                bne nosig		;not valid

		lda cyllow_g		;check ATA/ATAPI signature
		#lda_low_reg
		bne no_ata
		lda cylhi_g
		#lda_low_reg
		bne nosig

		lda status_g
		#lda_low_reg
                beq ok			;probably nothing here

		ldy #0
lp1     	dex
		bne jx
		dey
		beq ok
jx		jsr wait_a_bit
		lda status_g
		#lda_low_reg
                asl
                bpl lp1			;wait DRDY, but not forever
ok
		lda #ident_d
		jsr identify_drive
		bcs nosig

yes_hdd		lda #$01		;*yes* drive=HDD
		.byte SKIP2
yes_cd		lda #$80		;*yes* drive=CDROM
		.byte SKIP2
yes_floppy	lda #$C0		;*yes* drive=ZIP

		#ldx_l_ current_drive_exp
		sta drive_type_exp,x
                ora ident_capabilities_exp
                and #%10000010
                beq *+4
                lda #64
                sta drive_lba_exp,x
                #ldy_l_a ident_maxsector_exp
                lda ident_maxhead_exp
                #sbc1c
                jsr OP_setgeometry
		clc
		rts

nosig		sec
		rts

no_ata		eor #$14
		bne nosig		;check atapi signature
		lda cylhi_g
		#lda_low_reg
		eor #$eb
		bne nosig
		lda #atapi_reset_d
		jsr ata_command
                bcs nosig
		#stz_a low_reg
		sta feat_g		;for some atapi drives
		lda #atapi_ident_d
		jsr identify_drive
		bcs nosig

		lda ident_config_exp
		and #31
		cmp #5			;CDROM?
		beq yes_cd
					;Floppy/Unknown, but it may work (if sector size=512)
		ldy #249
		ldx #215
j11		iny
		beq yes_cd		;Looks like an old CD-ROM
j1		inx
		beq yes_floppy		;It's a floppy
		lda ident_model_number_exp-216,x
		cmp cdtext-250,y
		beq j11
cc		cpy #250
		beq j1
		dex
		dey
		gne cc

cdtext		.byte $43,$44,$2d,$52,$4f,$4d
		.pend
;---------------------------------------
; Identify Drive
; C=1 if error
;
; reads indentify info from drive
;---------------------------------------
identify_drive	.proc
		jsr ata_data_command
		bcs rrts		;error C=1

		lda #ram_exp_buffer0
		jsr OP_read512_here

		ldx #182		;fix ascii strings
cp		lda ident_serial_number_exp-182,x
		pha
		lda ident_serial_number_exp-181,x
		sta ident_serial_number_exp-182,x
		pla
		sta ident_serial_number_exp-181,x
		inx
		inx
		bne cp
		geq ata_pi_nomore_data
		.pend
rrts		rts
;---------------------------------------
; misc settings for existing drives
;
; Also sets the drive identify message
;--------------------------------------
OP_XIT_hdinit_2	.proc
		sta current_drive_exp
                jsr OP_get_current_drive_type
		beq rrts

		#xit_init

		jsr set_power_management_and_features
                sec
		gcs LWOP_reporterror_73_xit
		.pend
;--------------------------------------
; Tell an ata drive the disk geometry
;
OP_set_hdd_max	.proc
		jsr OP_setgeometry
		bmi err			;for chs only
                tya
		jsr ata_select_drive_A
		bcs err			;error C=1
                lda next_sector+3
		sta low_reg
		sta nsector_g
		lda #init_d
		gne ata_command
err		rts
		.pend

OP_setgeometry	.proc
		sty next_sector+3
		tay
		jsr OP_get_current_drive_type
		bmi err			;for hdd only

		lda next_sector+3
		sta drive_maxsector_exp,x
		lda next_sector+2
		sta drive_maxcylinderlo_exp,x
		lda next_sector+1
		sta drive_maxcylinderhi_exp,x
		tya
		sta drive_maxhead_exp,x
		asl
err		rts
		.pend
;--------------------------------------
ofs_fs_ident	.text "c64-ide v0.02-a "
cfs_fs_ident	.text "c64 cfs v 0.11b "
;--------------------------------------
; Try to identify filesystem on disk
;
OP_XIT_init_filesystem_from_disk .proc

		jsr OP_get_current_drive_type
		and #<~FS_MASK
		sta drive_type_exp,x	;Unknown filesystem
                pha
		#lda0			;First partition is active
		sta drive_workpartition_exp,x

                txa
                lsr
                ror			;*64
                ror
                sta partition_offset_exp
                tax
invali		#lda0
		sta partition_start_exp,x
		inx
                txa
		and #63
		bne invali

		pla
		bpl ata
		asl
                php

		lda #ATAPI_TEST_UNIT_READY
		jsr OP_atapi_packet
		jsr OP_atapi_command	;double ATAPI_SENSE_UNIT_ATTENTION after reset

                plp
		bpl isofs

		#lda0			;ZIP/LS120 drive, LBA sector 0
		.byte SKIP2
ata		lda #1			;ATA CHS (0/0/1)
		sta next_sector+3
		#stz_a next_sector
		#stz_aa next_sector+1
		#stz_aa next_sector+2

		lda #ram_exp_buffer0
		jsr XIT_read_a_sector

		ldx #15
lp1		lda boot_sector_ident_cfs_exp,x
		eor cfs_fs_ident,x
		bne inifi_nocfs
		dex
		bpl lp1

		lda boot_sector_default_p_cfs_exp
                pha
		#ldy_l_a boot_sector_maxsector_cfs_exp
		lda boot_sector_maxcyllow_cfs_exp
		sta next_sector+2
		lda boot_sector_maxcylhi_cfs_exp
		sta next_sector+1
		lda boot_sector_maxhead_cfs_exp
		and #%01001111
                pha
		jsr OP_set_hdd_max	;Tell disk the new geometry

		#cnext boot_sector_partdir1_cfs_exp , next_sector

		lda #ram_exp_buffer0
		jsr XIT_read_a_sector	;-=> error

		lda #ram_exp_buffer0
		sta pom2

		#ldx_l_a current_drive_exp;Store disk geometry
                pla
                and #64
                sta drive_lba_exp,x
                pla
		sta drive_workpartition_exp,x

		#ldx_l_a partition_offset_exp

loop		ldy #16
lp2		#lda_zp_y pom1
		sta partition_start_exp,x
                jsr inylda
		sta partition_end_exp,x
                jsr inylda
		sta partition_del_dir_exp,x
                jsr inylda
		sta partition_root_dir_exp,x
		sta partition_workdir_exp,x
		inx
		tya
		sec
		sbc #11
		tay
		and #3
		bne lp2
		lda pom1
		adc #32-1		;C=1
		sta pom1
		bne loop
		inc pom2
		txa
		and #63
		bne loop
;		#lda0
		jsr OP_check_partition_selection;Invalid? -=> error
                jsr LWOP_reporterror_77_xit
		lda #FS_CFS             ;CFS filesystem
                gne detectok

inylda		iny
		iny
		iny
		iny
		#lda_zp_y pom1
		rts
;----------------------------
inifi_nocfs	
		ldx #15
lp1c		lda boot_sector_ident_ofs_exp,x
		eor ofs_fs_ident,x
		bne inifi_noofs
		dex
		bpl lp1c

		#ldy_l_a boot_sector_maxsector_ofs_exp
		lda boot_sector_maxcyllow_ofs_exp
		sta next_sector+2
		lda boot_sector_maxcylhi_ofs_exp
		sta next_sector+1
		lda boot_sector_maxhead_ofs_exp
		and #%00001111		;no lba here!                
                jsr OP_set_hdd_max

		ldy #3
                lda #$0f
tue             ldx fk,y
		and boot_sector_root_ofs_exp,x
                sta szam,y
                lda #$ff
                dey
                bpl tue

		lda #%10000000		;Make partition valid, not hidden and readonly
                jsr szam_rootworkbig

		lda #FS_OFS             ;CFS filesystem
                gne detectok
                
fk		.byte 3,1,0,2
;----------------------------
jotyp		.byte $01,$04,$06,$0b,$0c,$0e,$0f
sectperclust	= pom4
rootdir		= pom5
datastart	= pom9

inifi_noofs	ldx #3
tu              #stz_a datastart,x
                dex
                bpl tu

		#ldx_l_a current_drive_exp
                #ldy_l_a drive_maxsector_exp,x
                lda drive_maxhead_exp,x
		jsr OP_set_hdd_max

ext		ldx #3
rp		lda datastart,x
                sta szam+5,x
                dex 
                bpl rp

		lda #>boot_sector_sign_exp
		jsr check_s
                bne inifi_nopc		;invalid

nu              ldy #<boot_sector_type_exp
                #lda_zp_y pom1
                ldx #6
lo              cmp jotyp,x
                beq fel
                dex
                bpl lo

nextone		jsr incc
                bne nu

nu2             #lda_zp_y pom1
                eor #$5
                beq fel
                jsr incc
                bne nu2
                geq trydirect 		;none found

incc            lda pom1
                clc
                adc #16			;search for partition entry
                and #63
                sta pom1
                rts

fel		cmp #1
                lda pom1
                pha
		php

		ldx #3
                ldy #<boot_sector_startp_exp
                clc
re              #lda_zp_y pom1
		adc szam+5,x
		sta next_sector,x
                sta datastart,x
                iny
                dex
                bpl re
                ora #$40
                sta next_sector
                
                #ldx_l_a current_drive_exp
                lda drive_lba_exp,x
                bne rpu

                jsr chs2lba
                lda szam+3
                and #$0f
                sta szam+3
                jsr lba2chs

rpu             plp
                php
		lda #ram_exp_fat
                bcs *+4
		lda #ram_exp_buffer0

		jsr XIT_read_a_sector	;read boot sector

                plp
                pla
                bcc ext
                pha

                jsr detectfat
                beq nomore
                
                lda #>boot_sector_sign_exp
                jsr init_pom1_y
                pla
                sta pom1
                #jmps nextone

trydirect	ldx #0
cp		lda ram_exp_buffer0_exp,x
                sta ram_fat_exp,x
		lda ram_exp_buffer0_exp+$100,x
                sta ram_fat_exp+$100,x
                inx
                bne cp
                
                jsr detectfat

                .byte SKIP1
nomore		pla

		lda partition_offset_exp
                and #63
                beq inifi_nopc		;no partitions?

		lda #FS_FAT             ;FAT filesystem
detectok       	#ldx_l_ current_drive_exp
		ora drive_type_exp,x
		sta drive_type_exp,x	
		jmp LWOP_reporterror_ok
;----------------------------
inifi_nopc     	sec
		gcs LWOP_reporterror_75_xit;Unknown filesystem
;----------------------------
isofs           #stz_a current_path
                #stz_aa current_path+1

                lda #ATAPI_READ_TOC
                jsr OP_atapi_packet
                #pc 9,$40
                #pc 7,>$800
                jsr OP_atapi_datain_command
                bcs unsupp
                lda #ram_exp_buffer0
                jsr OP_read2048_here
		jsr ata_pi_nomore_data
                bcs unsupp
                lda toc_first_session_number_exp
                eor toc_last_session_number_exp
                beq unsupp
                ldx #3
                lda #15
lu              adc toc_last_session_lba_exp,x
                sta next_sector,x
                #lda0
                dex
                bpl lu
                gmi xi

unsupp          lda #15
		sta next_sector+3
		#stz_a next_sector
		#stz_aa next_sector+1
		#stz_aa next_sector+2

xi              inc current_path
		bmi ene
                ldx #3
ll      	inc next_sector,x
                bne xi2
                dex
		bpl ll
ene		lda current_path+1
		bne detectokx
                sec
                gcs LWOP_reporterror_75_xit;filesystem not found

xi2     	lda #ram_exp_buffer0
		jsr XIT_read_a_sector

                ldx #4
ly              lda cdstr,x
                cmp isofs_vd_id_exp,x
                bne xi
                lda isofs_vd_root_directory_exp,x
		sta szam,x
                dex
                bpl ly
                
                #ldy_l_a isofs_vd_type_exp
                beq xi
                iny
                beq ene		;0xff = end
		cpy #4
                bcs xi
                cpy #2
                php

		lda #%10000000		;Make partition valid, not hidden and readonly
                jsr szam_rootworkbig

		lda #FS_JOLIET
                plp
detectokx       bne detectok            ;Joliet filesystem

                lda #FS_ISO
		sta current_path+1	;ISO filesystem
                gne xi			;retry

cdstr           .text "cd001"
;----------------------------
szam_rootworkbig pha
		#ldx_l_a partition_offset_exp
		ldy #3
                sec
luk             lda szam,y
		sta partition_root_dir_exp,x
		sta partition_workdir_exp,x
                inx
                dey
                bpl luk
                dex
                pla
		sta partition_start_exp-3,x
		lda #%00010000
		sta partition_end_exp-3,x
not             rts
;----------------------------
detectfat      	lda #>boot_sector_fat_sign_exp
		jsr check_s
                bne not

                lda boot_sector_fat_bytepersec_exp+1
                eor #2
                ora boot_sector_fat_bytepersec_exp
		bne not

                ldx #<boot_sector_fat_reserved_exp
                jsr breadword0
                lda boot_sector_fat_secperclust_exp
                sta sectperclust

                lda #3
                tay
		ora partition_offset_exp
                tax

                clc
lel             lda datastart,y
                adc next_sector,y
                sta partition_fat_fat_exp,x
                sta rootdir,y
                dex
                dey
                bpl lel

                ldx #<boot_sector_fat_fatsectors_exp
                jsr breadword0
                php
                bne lel2x

                ldx #<boot_sector_fat_fat2sectors_exp
                jsr bcopydword

lel2x           ldx #3
                clc
lel2            lda rootdir,x
                adc next_sector,x
                sta rootdir,x
                dex
                bpl lel2
		#dec_l_a boot_sector_fat_numfat_exp
                bne lel2x

                ldx #<boot_sector_fat_rootcount_exp
                jsr breadword0

                ldx #4
nn              lsr next_sector+2
                ror next_sector+3
                dex
                bne nn

		ldx #3
                clc
lel4		lda next_sector,x
                adc rootdir,x
                sta datastart,x
                lda boot_sector_fat_rootclust_exp,x
                sta szam,x
                dex
                bpl lel4

		plp
                beq foundfat		;fat32

		ldx #<boot_sector_fat_totalsect_exp
		jsr breadword0
                bne rdr
                ldx #<boot_sector_fat_totalsect2_exp
                jsr bcopydword
rdr
                ldx #3
                sec
hhe             lda next_sector,x
                sbc datastart,x
                sta next_sector,x
                dex
                bpl hhe

		lda sectperclust
cc              lsr
                beq tpi
	      	lsr next_sector
	      	ror next_sector+1
	      	ror next_sector+2
	      	ror next_sector+3
                #jmps cc

const4085	.byte 0,0,>4085,<4085

tpi		ldx #3
qj      	lda next_sector,x
                sbc const4085,x
                #stz_a szam,x
                dex
                bpl qj
                
                lda #1
                sta szam		;fat16

                bcs foundfat
                lda #2                  ;fat12

foundfat	pha

		ldy #2
lel6            lda sectperclust
                eor #$ff
                ldx #3
                sec
                .byte $2c
lel5		lda #$ff
		adc datastart,x
                sta datastart,x
                dex
                bpl lel5
                dey
                bne lel6

                lda #%10000000		;valid, readonly
                jsr szam_rootworkbig
                lda sectperclust
                sta partition_fat_sectperclust_exp-3,x
                lda boot_sector_fat_rootcount_exp
                sta partition_fat_rootsect_exp-3,x
                lda boot_sector_fat_rootcount_exp+1
                sta partition_fat_rootsect_exp-2,x
		pla
                sta partition_fat_type_exp-3,x

                ldy #3
tq              lda datastart,y
                sta partition_fat_data_exp,x
                lda rootdir,y
                sta partition_fat_root_dir_exp,x
                dex
                dey
                bpl tq

		txa
		clc
                adc #5
                sta partition_offset_exp
                and #31
                rts

breadword0	clc
bcopydword      ldy #3
jc		lda boot_sector_fat_exp,x
                sta next_sector,y
                tya
                eor #2
                beq jf
jf2             inx
                dey
                bpl jc
                rts

jf		bcs jf2
		sec
		sta next_sector
		sta next_sector+1
		lda next_sector+2
		ora next_sector+3
		rts

check_s	      	jsr init_pom1_y

                ldy #<boot_sector_sign_exp
                lda #$55
                #cmp_zp_y pom1
                bne nke
                iny
                asl
                #cmp_zp_y pom1
nke		rts

		.pend
;----------------------------
;result in Y
;
OP_check_partition_selection .proc
		sec
		sbc #1
		bcs fe
		#ldx_l_a current_drive_exp
		lda drive_workpartition_exp,x

fe		tay
		cpy #16
		bcs invalid
		lsr
		ror
		ror
		ror
		ora current_drive_exp
		ror
		ror
		ror
		tax
		lda partition_start_exp,x
		bpl invalid		;Must be valid
		lda partition_end_exp,x
		and #%10110000
		eor #%00010000		;CFS?
		beq valid
invalid		iny
		#tyx_a
		ldy #0
                sec
                rts
valid		clc
		rts
		.pend
;--------------------------------------
kill_drives	.proc
		#lda0
		ldx #DRIVES-1
lp		sta drive_type_exp,x
		dex
		bpl lp
		rts
		.pend
;---------------------------------------
set_power_management_and_features .proc
		jsr select_drive

		lda #freeze_lock_d	;Freeze lock
		jsr ata_command

                lda #read_native_max_address_d
                jsr ata_command
                bcs non

                #lda0                   ;Disable HPA
                jsr setmax
non
                lda #4			;Set max freeze lock
                jsr setmax

		lda current_drive_exp
                lsr			;master/slave?
                .if DRIVES=4
                tax
                lda setup_powerman_exp,x
		.else
		lda setup_powerman_exp
                .fi
		bcc master
		lsr
		lsr
		lsr
		lsr
master		and #%00001111
		beq def
		tax
		lda powerconst-1,x
		sta low_reg
		sta nsector_g

		lda #idle1_d		;Power management feature
		jsr ata_command

		lda #idle2_d		;Power management feature
		jsr ata_command

def		ldy #0                  ;retry
		jsr set_features

                ldy #3                  ;writecache
		jsr set_features

                ldy #6                  ;readahead
		jsr set_features

		jsr OP_get_current_drive_type
                bpl exit
                asl
		bmi exit		;not cdrom

		lda #ATAPI_SET_SPEED
		jsr OP_atapi_packet

                ldy #9                  ;cdromspeed
		jsr afeat

		#pw 2                   ;255Kb/s should be enough for everyone
		#pc 3,$ff
		jsr OP_atapi_command
		bcc jo
		jsr OP_atapi_command	;retry
jo
		jsr cdrom_idleoff
		
exit		rts

featc		.byte %10000000,$33,$99		;retry
		.byte %01000000,$82,$02		;writecache
                .byte %00100000,$55,$aa         ;readahead
                .byte %00010000,$ff,$01         ;cdromspeed

afeat		lda current_drive_exp
		lsr	    		;master/slave?
                .if DRIVES=4
                tax
                .fi

		lda featc,y
		bcs nm
		lsr
                lsr
                lsr
                lsr
nm		.if DRIVES=4
		and setup_features_exp,x
                .else
		and setup_features_exp
                .fi
		beq dis
                iny
dis		lda featc+1,y
		rts

setmax		sta low_reg
		sta feat_g
		sta nsector_g
                lda #set_max_d
                gne fdf
		
set_features	jsr afeat
		sta low_reg
		sta feat_g
		lda #setfeatures_d
fdf		gne ata_command

powerconst	.byte 0,12,24,36,48,60,72,84,96,120,180,240,241,242,244
		.pend
