;
; misc.asm - misc stuff
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;------------------------------
;Used by #devicehandler macro
;------------------------------
		#opencfg_start
set_current_drive_from_drive_number .proc
		tay
                beq cdrvunk		;0=unknown
		ldx #0
cdrvlp		cmp current_drivenumbers_exp,x
		beq cdrvfnd
		inx
                cpx #current_drivenumbers_end_exp-current_drivenumbers_exp
		bcc cdrvlp
cdrvunk		ldx #DRIVES+2		;DRIVES+2 unknown
cdrvok		.if SUPERCPU=1
		txa
		asl
		tax
		.fi
		rts

cdrvfnd		txa
		cmp #DRIVES+2		;>DRIVES+2 floppy 8-11
		bcc cdrvnf
		adc #7-DRIVES-2
		tay
		gne cdrvunk

cdrvnf		cmp #DRIVES		;DRIVES = pc DRIVES+1 = lcd
		bcs cdrvok
		sta current_drive_exp
		lda drive_type_exp,x	;Not existing IDE drive
		beq cdrvunk
		ldx #DRIVES-1		;DRIVES-1 = ide
		gcc cdrvok
		.pend
		#opencfg_end

exit_file_not_found .proc
		lda #KERNAL_FILE_NOT_FOUND
		.byte SKIP2
		.pend
exit_device_not_present .proc
		lda #KERNAL_DEVICE_NOT_PRESENT
		.byte SKIP2
		.pend
exit_not_input_file .proc
		lda #KERNAL_NOT_INPUT_FILE
		.byte SKIP2
		.pend
exit_not_output_file .proc
		lda #KERNAL_NOT_OUTPUT_FILE
		.byte SKIP2
		.pend
exit_missing_filename .proc
		lda #KERNAL_MISSING_FILENAME
		.byte SKIP2
		.pend
exit_illegal_device .proc
		lda #KERNAL_ILLEGAL_DEVICE_NUMBER
		.byte SKIP2
		.pend
exit_file_data	.proc
		lda #KERNAL_FILE_DATA
		.byte SKIP2
		.pend
exit_out_of_memory
		lda #KERNAL_OUT_OF_MEMORY
exit_kernal_error
		pha
		#psr $f719
		jmp r_clrchn
;------------------------------
;Guess what ;) (usefull for clock reading)
;------------------------------
iny_bcd2bin     .proc
		iny
bcd2bin		.proc
		pha
		lsr
		lsr
		lsr
		lsr
		tax
		pla
		cpx #0
		beq iybmhdy
jkq		sbc #6
		dex
		bne jkq
iybmhdy		rts
		.pend

newchrgot	.proc
		dec $01
		jsr $79
		php
		inc $01
		plp
		rts
		.pend

newchrget	.proc
		dec $01
		jsr $73
		php
		inc $01
		plp
		rts
		.pend

OP_cfg16up	.proc			;Go to cfg_16, but only replace $a000-$bfff
		lda $01
		sta help_01_exp
                and #$f8
		ora #$06
		sta $01
		sta cfg_16
		rts
		.pend

OP_cfg16down	.proc			;Come back from replacing $a000-$bfff
		sta cfg_open
		lda help_01_exp
		sta $01
		rts
		.pend

		.if SUPERCPU=1
OP_scpu01up	.proc
		lda $01
		sta help_01_scpu_exp
		ora #$07
		sta $01
		rts
		.pend

OP_scpu01down	.proc
		lda help_01_scpu_exp
		sta $01
		rts
		.pend
		.fi

                .if SUPERCPU=1
scpu_default_opt .proc
		php
                sei
		sta scpu_hwenable
                sta scpu_nooptim	;default optimization settings
                lda #%11000001
                sta scpu_optimv2
                sta scpu_hwdisable
                plp
                rts
                .pend
                .fi

                .if SUPERCPU=1
scpu_no_opt	.proc
		php
                sei
                sta scpu_hwenable
                sta scpu_nooptim	;no optimization, sprites!!!
                sta scpu_hwdisable
                plp
                rts
                .pend
                .fi

		.if SERIAL_HANG_FIX=1
test_hang	.proc
		pha
		#phx_a
		ldx #0
		lda $dd00
		and #$df
		sta $dd00
mn		lda $dd00
		asl
		bcs vr
		lda $d011
		bpl *-3
		lda $d011
		bmi *-3
		dex
		bne mn
		lda #$80
		sta Status
vr		#plx_a
		pla
		rts
                .pend
		.fi
;----------------------------------
;Print hex number
;
NO_printhx	.proc
		pha
		lsr
		lsr
		lsr
		lsr
		jsr ijou
		pla
		and #15
ijou		cmp #10
		bcc *+4
		adc #6
		adc #$30
		jmp newchrout_t
		.pend
;----------------------------------
;Check RUN/STOP Z=1 if pressed
;
check_stop	.proc
		lda StopFlag
		eor #$7f
		bne *+4
		sta $c6
		rts
		.pend
;---------------------------------------
;Print imediate
;---------------------------------------
LWNO_print_imediate .proc
		pla
		sta $22
		pla
		jsr printall
                #psz $22
		rts
		.pend
;---------------------------------------
;Print A/Y
;---------------------------------------
LWNO_print_ay	.proc
		sta $22
		sty $23
		ldy #0
		geq printall.cbv
;---------------------------------------
;Printing common
;---------------------------------------
printall	.proc
		sta $23
		sty $24
		ldy #0
svc		inc $22
		bne cbv
		inc $23
cbv		lda ($22),y
		beq prntki
		jsr newchrout_t		; chrout
		gcc svc
prntki		ldy $24
		rts
		.pend
;--------------------------------------
;make drive number to default
;--------------------------------------
OP_hdinit_drivenumbers .proc
		ldx #(current_drivenumbers_end_exp-current_drivenumbers_exp)/2
lp		lda setup_drivenumbers_exp-1,x
		pha
                and #$0f
                beq jq
                cpx #DRIVES+1
                bne nq
                sbc #4
nq              clc
                adc #7
jq		sta current_drivenumbers_exp-1,x
                pla
                lsr
                lsr
                lsr
                lsr
                beq jq2
                clc
                adc #7
jq2             sta current_drivenumbers_exp-1+(current_drivenumbers_end_exp-current_drivenumbers_exp)/2,x
		dex
		bne lp
		lda setup_boot_drive_exp
		sta ActualDev
		rts
		.pend

buffer_subpart_cp .proc
		#ldx_l_ buffer_currentp_exp
buffer_subpart_cp2 .proc
		ora buffer_start,x
		sta current_path+1
		.if SUPERCPU=1
		ldy #1
		sty current_path+2
		dey
		.else
		ldy #0
		.fi
		sty current_path
		rts
		.pend

buffer_tree_pom1 .proc
		#ldx_l_a buffer_currentp_exp
		lda buffer_filepos2_exp,x
		asl
		sta pom1
                lda #(>b_cfs_file_tree_exp)/2
                rol
		ora buffer_start,x
		sta pom2
		.if SUPERCPU=1
		lda #1
		sta pom3
		.fi
		rts
		.pend
;----------------------------------------
;Increment high part of filepos
;
incfileposhi2048 .proc
		lda #>2048
                .byte SKIP2
incfileposhi512	.proc
                lda #>512
		#ldx_l_ buffer_currentp_exp
		clc
		adc buffer_filepos2_exp,x
		sta buffer_filepos2_exp,x
		bcc incfileposhi.rt

incfileposhi	.proc
		.if SUPERCPU=1
                xba
                lda buffer_filepos3_exp,x
                inc a
                sta buffer_filepos3_exp,x
                bne fw3
                lda buffer_filepos4_exp,x
                inc a
                sta buffer_filepos4_exp,x
fw3             xba
                .else
		inc buffer_filepos3_exp,x
		bne fw3
		inc buffer_filepos4_exp,x;!!!!!
fw3		
		.fi
rt		rts
		.pend
;-------------------------
;Save or restore allocator state
;
restore_file_state .proc
		clc
		.byte SKIP1
save_file_state .proc
		sec
		php
		lda #>b_cfs_file_state_exp
		jsr buffer_subpart_cp
		ldy #<(b_cfs_file_state_exp+find_free_backup_end_exp-cfs_file_info_backup_start_exp-1)
		ldx #find_free_backup_end_exp-cfs_file_info_backup_start_exp-1
in		plp
cwe		lda cfs_file_info_backup_start_exp,x
		bcs vs
		#lda_zp_y current_path
                .byte SKIP2
vs		#sta_zp_y current_path
		sta cfs_file_info_backup_start_exp,x
		dey			;restore allocator state
		dex
		bpl cwe
		rts
		.pend

fileopprepare   .proc
		lda #INFO_IGNORE	;be safe
		sta buffer_info_exp,x
in		lda buffer_drive_exp,x
		sta current_drive_exp
                lda buffer_partition_exp,x
                sta partition_currentp_exp
		rts
		.pend
